#ifndef LEAF_IR_BLOCK_HPP
#define LEAF_IR_BLOCK_HPP

#include "fwd.hpp"

#include <vector>

namespace ir
{

struct block_seq_item
{
	//one of these items is defined
	shared_ptr<instruction> an_instruction;
	shared_ptr<block> a_block;
	
	block_flow flow;
	
	block_seq_item( shared_ptr<instruction> & i, block_flow flow )
		: an_instruction( i ), flow( flow ) { }
	block_seq_item( shared_ptr<block> & b,  block_flow flow )
		: a_block( b ), flow( flow ) { }
};

struct block
{
	function & owner_function;

	std::vector<block_seq_item> items;	
	
	//will be null if there is no error_block
	shared_ptr<block> error_block;
	int error_level;
	
	void push_back( shared_ptr<instruction> i, block_flow flow )
	{ items.push_back( block_seq_item( i, flow ) ); }
	void push_back( shared_ptr<block> b, block_flow flow )
	{ items.push_back( block_seq_item( b, flow ) ); }

	void append( block & o );
	
	block( function & owner )
		: owner_function( owner )
	{ }

	shared_ptr<block> insert_block()
	{
		auto s = make_shared<block>(owner_function);
		push_back( s, block_flow::sequential );
		return s;
	}
	
	shared_ptr<value> make_temp( shared_ptr<type const> dt );
	
	void validate() const;
};

} //eon ir

#endif