#ifndef LEAF_IR_COMPILER_LEAF_HPP
#define LEAF_IR_COMPILER_LEAF_HPP

#include "compiler.hpp"

#include "../lang/fwd.hpp"
#include "../lang/scope_ref.hpp"

namespace ir
{

/**
	Leaf specific compiler (translates from Leaf code to Leaf-IR code)
*/
class compiler_leaf : public compiler
{
	friend class type_manager_leaf;
	
	shared_ptr<leaf::scope> globals;
	shared_ptr<type_manager_leaf> types;

	virtual type_manager & get_types();
public:
	compiler_leaf( shared_ptr<leaf::scope> globals );

	shared_ptr<ir::module> compile_expr( leaf::expression const & expr );
	shared_ptr<ir::module> compile_block( leaf::statement_block const & bk );
	shared_ptr<ir::module> compile_full( leaf::statement_block const ** bks, size_t len );

protected:
	enum eval_flags
	{
		//we are binding to this value
		ef_bind = 0x1,
		//this value is being directly called as a function
		ef_calling = 0x2,
	};
	value_ref eval_single_expr( leaf::expression const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref_vector_t eval_list_expr( leaf::expression const & expr, builder & where, unsigned eval_flags = 0 );
	void eval_list_expr( value_ref_vector_t & out, leaf::expression const & expr, builder & where, unsigned eval_flags = 0 );
	void eval_funccall_expr( value_ref_vector_t & out, leaf::expr_funccall const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_subscript( leaf::expr_subscript const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_field( leaf::expr_field const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_special_field( leaf::expr_special_field const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_transform( leaf::expr_transform const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_transform_bind_func( leaf::expr_transform const & expr, builder & where );
	value_ref eval_binary( leaf::expr_binary const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_unary( leaf::expr_unary const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_conditional( leaf::expr_conditional const & expr, builder & where, unsigned eval_flags = 0 );
	value_ref eval_text( leaf::expr_text const & expr, builder & where );
	value_ref eval_switch( leaf::expr_switch const & expr, builder & where );
	
	void build_expression_statement( leaf::expression_statement const & expr, builder & where );
	void build_block( leaf::statement_block const & bk, builder & where );
	void build_statement( leaf::statement const & st, builder & where );
	void build_init_statement( leaf::init_statement const & vs, builder & where );
	void build_do_statement( leaf::do_statement const & on, builder &  where );
	value_ref build_function( leaf::expr_funcdefn const & expr, builder & where );
	void build_defer_statement( leaf::defer_statement const & ds, builder & where );
	void build_return_statement( leaf::return_statement const & ds, builder & where );
	void build_fail_statement( leaf::fail_statement const & fs, builder & where );
	void build_loop_flow_statement( leaf::loop_flow_statement const & ls, builder & where );
	builder maybe_build_on_error( leaf::error_statement const & es, builder & where );
	void build_type_statement( leaf::type_statement const & ts, builder & where );
	void build_multi_statement( leaf::multi_statement const & ms, builder & where );
	
	void build_assign_statement( leaf::assign_statement const& as, builder &  where );

	void initialize_leaf( value_ref ptr, leaf::extr_type_ref const & init_type, 
		shared_ptr<leaf::expression> value, unsigned flags, builder & where );

	value_defn obtain_symbol_value_defn( leaf::scope_ref sr );
};

} //eon ir
#endif