#ifndef LEAF_IR_DUMP_HPP
#define LEAF_IR_DUMP_HPP

#include "fwd.hpp"
#include "../util/dump_base.hpp"

#include <set>

namespace ir
{

/**
	NOTE: A pointer is used instead of a reference to allow for debugging of uninit'd
	values.
*/
class dump : public leaf_util::dump_base
{
	bool show_all_types;
public:
	dump( std::ostream & out );
	dump() { }
	
	void set_show_all_types( bool on ) 
	{ show_all_types = on; }
	
	void write( module const * m );
	void write( function const * f );
	void write( type const * t );
	void write( shared_ptr<type const> t )
	{ write( t.get() ); }
	void write_global_variable( value const * gv );
	void write_local_variable( value const * lv );
	
	void write_instruction( instruction const * i );
	
	void write_value_ref( value_ref vr, bool suppress_type = false );
	void write( value_ref vr )
	{ write_value_ref( vr ); }
	
	void write_value_defn( value_defn vd, bool suppress_type = false );
	void write( value_defn vd )
	{ write_value_defn( vd ); }
	
	void write_block( block const * );
	
	static std::string get_type( type const * t );
private:
	template<typename T>
	void write_value_list( std::vector<T> const & values );
	template<typename T>
	void write_tuple( std::vector<T> const & values );

	void write_block_seq_item( block_seq_item const & si );
	
	#define DUMP_WRITE_INSTRUCTIONS \
		WINST(acquire_shared) \
		WINST(alloc_array) \
		WINST(alloc_shared) \
		WINST(array_offset) \
		WINST(array_ptr_offset) \
		WINST(binary) \
		WINST(bind_func) \
		WINST(bind_func_explicit) \
		WINST(bind_raw_func) \
		WINST(cast) \
		WINST(clone) \
		WINST(comment) \
		WINST(fail) \
		WINST(free_array) \
		WINST(funccall) \
		WINST(loop) \
		WINST(loop_flow) \
		WINST(malloc) \
		WINST(pack_tuple) \
		WINST(promote) \
		WINST(loss) \
		WINST(release_shared) \
		WINST(resolve) \
		WINST(resume) \
		WINST(return) \
		WINST(special_field) \
		WINST(store) \
		WINST(struct_pointer) \
		WINST(switch) \
		WINST(unary) \
		WINST(void_context)
	
	#define WINST(type) \
		void write_##type##_inst( type##_inst const& inst );
		DUMP_WRITE_INSTRUCTIONS
	#undef WINST
	
	void write_promote_loss_inst( promote_loss_inst const & inst, char const * label );
	void write_shared_object_defn( shared_object_defn const & sod, char const * op );
	
	void write_value_name_part( value const * v, bool supress_type = false );
	void write_value_type_part( value const * v, bool supress_type = false );
	void write_value( value const * v, bool suppress_type = false )
	{
		write_value_name_part( v, suppress_type );
		write_value_type_part( v, suppress_type );
	}
	
	//track all values for which the type has been shown
	std::set<value const*> shown_type;
	
	template<typename T>
	void validate( T const & o );
};

} //eon ir

#endif