#ifndef LEAF_IR_TYPE_MANAGER_HPP
#define LEAF_IR_TYPE_MANAGER_HPP

#include "fwd.hpp"

#include <set>
#include <map>

namespace ir
{

/**
	For efficiency the type_manager tries to merge all equivalent types. Higher level code
	should not assume this however as other things make "type" instances as well. In practice
	it was too troublesome to ensure each unique type has only one instance.
*/
class type_manager
{
	shared_ptr<type const> type_boolean, type_empty;
	std::map<int,shared_ptr<type const>> integer_cache, binary_cache;

protected:
	struct type_key {
		bool shared, optional, lvalue, value_ptr, array_ptr;
		intptr_t cref;
		shared_ptr<type const> cached;
		
		bool operator<( type_key const & o ) const {
			if( cref != o.cref )
				return cref < o.cref;
			if( lvalue != o.lvalue )
				return lvalue < o.lvalue;
			if( shared != o.shared )
				return shared < o.shared;
			if( optional != o.optional )
				return optional < o.optional;
			if( value_ptr != o.value_ptr )
				return value_ptr < o.value_ptr;
			if( array_ptr != o.array_ptr )
				return array_ptr < o.array_ptr;
			return false;
		}
	};
	std::set<type_key> known_type_refs;

public:
	type_manager();
	
	shared_ptr<type const> get_boolean()
	{ return type_boolean; }
	shared_ptr<type const> get_integer( int bitsize = 0 );
	shared_ptr<type const> get_binary( int bitsize = 0 );
	shared_ptr<type const> get_empty()
	{ return type_empty; }
};

} //eon ir

#endif