#ifndef DUMP_HPP
#define DUMP_HPP

#include "fwd.hpp"
#include "../util/dump_base.hpp"

#include <string>
#include <set>

namespace leaf
{

struct extr_type_ref;

/**
	Converst this into a standard format suitable for checking that
	the parser actually did the right thing. This doesn't generate parsable code
	since we wish to dump additional information. It should however look close
	to real code.
*/
class dump : public leaf_util::dump_base
{
public:
	dump( std::ostream & out );
	dump();

	void set_show_all_types( bool on ) 
	{ show_all_types = on; }
	void set_validate( bool on )
	{ do_validate = on; }
	void set_show_full_type( bool on )
	{ show_full_type = on; }
	void set_show_spec_type( bool on )
	{ show_spec_type = on; }
	void set_hide_var_spec_no_fundamental( bool on )
	{ hide_var_spec_no_fundamental = on; }
	void set_keep_variant( bool on )
	{ keep_variant = on; }

	//allow showing implicit settings like default/unset, default is false
	void set_show_spec_implicit( bool on )
	{ show_spec_implicit = on; }
	
	void write_intr_type( extr_type_ref const & tr ) //TODO: change name
	{ dump_intr_type( tr ); }
	void write_intr_type( intr_type const * vt, bool expand_defined = false );
	void write_expression( expression const & ex )
	{ dump_expression( &ex, true ); }
	void write_type_spec( type_spec const & vt )
	{ dump_type_spec( &vt ); }
	void write_type_spec( shared_ptr<type_spec const> vt )
	{ dump_type_spec( vt.get() ); }
	void write_var_statement( var_statement const & vs )
	{ dump_var_statement( &vs ); }
	void write_statement_block( statement_block const & block ) { 
		dump_statement_block( &block ); 
		dump_outline_types();
	}
	void write_statement( statement const & st ) { 
		dump_statement( &st ); 
	}
	void write_var_decl( var_decl const & vd ) {
		dump_var_decl( &vd );
	}
	
	static std::string get_extr_type( extr_type_ref const & vt );
	static std::string get_type_spec( type_spec const & vt );
	static std::string get_expression( expression const & expr );
private:
	void dump_outline_types();
	
	void dump_assign_statement( assign_statement const * as );
	void dump_defer_statement( defer_statement const * defer );
	void dump_do_statement( do_statement const * ds );
	void dump_expression_statement( expression_statement const * es );
	void dump_fail_statement( fail_statement const * fs );
	void dump_for_statement( for_statement const * fs );
	void dump_import_statement( import_statement const * is );
	void dump_init_statement( init_statement const * is );
	void dump_intr_type( extr_type_ref const & tr, bool expand_defined = false );
	void dump_intr_type( intr_type const & tr, std::vector<std::string> & markers, bool expand_defined = false );
	void dump_loop_flow_statement( loop_flow_statement const * ls );
	void dump_multi_statement( multi_statement const * ms );
	void dump_noop_statement( noop_statement const * ns );
	void dump_return_statement( return_statement const * rs );
	void dump_statement_block( statement_block const * block );
	void dump_statement( statement const * st );
	void dump_tuple( expr_with_sub const * ex, char open, char close, char sep );
	void dump_type_spec( type_spec const * vt );
	void dump_type_statement( type_statement const * ts );
	void dump_var_decl( var_decl const * vd );
	void dump_var_statement( var_statement const * vs );
	
	void dump_error_statement( error_statement const & ds );
	void dump_expression( expression const * ex, bool isolated = true );

	bool show_all_types, do_validate, show_full_type, show_spec_type, hide_var_spec_no_fundamental,
		keep_variant, show_spec_implicit;
	//track all items for which the type has been shown
	std::set<void *> shown_type;
	
	std::set<intr_type const*> outline_types;

	//show either type or spec_type depending on mode
	void show_type( extr_type_ref type, type_spec const * spec );
	
	template<typename T>
	void dump_intr_type_tuple_inner( char o, char c, T const * vt );
	void _dvtti_sub( shared_ptr<type_spec const> const & vt )
	{ dump_type_spec( vt.get() ); }
	void _dvtti_sub( type_spec const * vt )
	{ dump_type_spec( vt ); }
	void _dvtti_sub( extr_type_ref const & vt )
	{ dump_intr_type( vt ); }
	
	template<typename T>
	void validate( T const & o );
};

} //eon leaf

#endif