#ifndef LEAF_SCOPE_REF_HPP
#define LEAF_SCOPE_REF_HPP

#include "core_types.hpp"

namespace leaf
{

class var_decl;
class scope;

/**
	A reference to an entity within the scope.
*/
struct scope_ref
{
	/**
		How far up in the parent chain this variable exists. 0 = current local scope (this is not
		the same as the current scope, since a continues/extends base does not increment
		this number)
	*/
	unsigned parent_level;
	/**
		Level from the root along non-continues axis (that is, incremented for each parent/extends
		scope level from the root).
	*/
	unsigned root_depth;
	
	//if not set then typing not done
	shared_ptr<var_decl> decl;
	
	//used privated by scope
	intptr_t scope_src_decl;
	
	scope_ref()
		: scope_src_decl( 0 )
	{ }
	
	void validate() const;
	bool is_set() const;
};

} //eon leaf

#endif