#ifndef STATEMENT_BLOCK_HPP
#define STATEMENT_BLOCK_HPP

#include "core_types.hpp"
#include "statement.hpp"
#include "fwd.hpp"

namespace leaf
{

/**
	A block of statements with an associated scope.
*/
struct statement_block
{
	statement_block( scope_style_t scope_style );
	~statement_block();
	
	shared_ptr<scope> symbols; //never null
	std::vector<shared_ptr<statement> > list;

	/**
		Update the "flow" field from the list of blocks. Called from the type after the
		blocks have been typed.
	*/
	void update_flow();
	
	unsigned get_flow() const
	{ return flow; }
	
	//usable by compiler
	mutable intptr_t cref;
	
	void validate() const;

	bool is_complete() const;
private:
	//all flow which leaves this block
	unsigned flow;
};

} //eon leaf

#endif