#ifndef LANG_TYPE_IDENTIFIER_HPP
#define LANG_TYPE_IDENTIFIER_HPP

#include "fwd.hpp"
#include "cerr.hpp"
#include "intr_type.hpp"

namespace leaf
{

class type_identifier
{
	scope & sc;
public:
	type_identifier( scope & sc )	
		: sc( sc )
	{ }

	typedef cerr_impl<extr_type_ref> resolve_t;
	typedef cerr_impl<shared_ptr<type_spec>> resolve_spec_t;

	/**
		Expands the type_spec to remove any unknown symbols and complete
		the other elements.
		
		@param to The target spec. This may be empty or already contain a partial spec
		@param from the source spec
	*/
	cerr_base expand_spec( type_spec & to, type_spec const & from );
	/**
		Expands provided type spec in place.
	*/
	cerr_base expand_spec( type_spec & cvt );
	
	enum determine_flags {
		//the type is sub-part of a higher type (does not apply to value-lists)
		df_sub_type = 1<<0,
	};
	
	/**
		Translates an input source type into a fully expanded and completed type.
	*/
	resolve_t determine( type_spec const & vt, unsigned flags = 0 );
	resolve_t determine( shared_ptr<type_spec const> vt, unsigned flags = 0 );
	
	/**
		Determines what the inferred type of the given type should be.
		@param spec the type specified (may be null)
		@param assign which type is being assigned to the "spec" var
		@param init_refer indicates infer is being done for reference initialization
	*/
	resolve_t infer( type_spec const & spec, extr_type_ref const & assign, bool init_refer );
	resolve_t infer( shared_ptr<type_spec const> spec, extr_type_ref const & assign, bool init_refer );
	resolve_spec_t infer_spec( type_spec const & spec, extr_type_ref assign, bool init_refer );
	
private:
	shared_ptr<intr_type const> resolve_type_name( variable_context const & where, 
		std::string const & name );
	shared_ptr<intr_type const> resolve_type_name( std::string const & name );
};


} //eon leaf

#endif