#ifndef LEAF_VAR_DECL_HPP
#define LEAF_VAR_DECL_HPP

#include "core_types.hpp"
#include "intr_type.hpp"

namespace leaf
{

struct intr_type;
struct type_spec;

struct var_decl
{
	enum form_t : int8_t
	{
		//declaring a variable with a single mapping
		f_single,
		//overloading a variable (implicit declaration)
		f_multi,
		//implicit var_statement for a function parameter
		f_param,
		//implict var_statement for a function return
		f_return,
		//capture variable from previous scope
		f_capture,
	} form;

	//import annotation/style
	std::string import_name;
	enum import_type_t : int8_t {
		//not an import
		it_none,
		//an ABI code import
		it_abi_code,
		//an ABI data import,
		it_abi_data
	} import_type;
	//name of module for the import (blank if undefined -- linker resolved)
	std::string import_module;
	//name mangling for the import/export
	enum name_form_t : int8_t {
		//import_name is the exact name
		nf_bare,
		//module/name are Leaf style
		nf_leaf,
	} name_form;
	
	//this symbol should be exported
	bool is_export;
	
	//the variable symbol
	std::string symbol;
	//the type of this variable (as provided by the statement)
	extr_type_ref type;
	shared_ptr<type_spec> spec_type;

	//used within the typer to get at original code, may be null
	shared_ptr<statement> source_statement;
	//the scope in which this decl appears (may be zero prior to adding)
	scope * var_scope;
	
	//usable by compiler/interpreter to create a map (initially 0)
	mutable intptr_t cref;
	mutable intptr_t xref;

	var_decl() {
		cref = 0;
		xref = 0;
		is_export = false;
		import_type = it_none;
		name_form = nf_bare;
		var_scope = nullptr;
	}
	var_decl( var_decl const & o ) {
		cref = 0;
		xref = 0;
		is_export = o.is_export;
		import_type = o.import_type;
		form = o.form;
		import_name = o.import_name;
		import_type = o.import_type;
		import_module = o.import_module;
		name_form = o.name_form;
		type = o.type;
		var_scope = o.var_scope;
	}
	
	bool is_first_class() const {
		return form == f_single || form == f_multi;
	}
	
	void validate() const;
};

} //eon leaf

#endif