#ifndef LANG_VARIABLE_CONTEXT_HPP
#define LANG_VARIABLE_CONTEXT_HPP

#include "scope_ref.hpp"
#include "fwd.hpp"

namespace leaf
{

/**
	Abstracts access to the scope for variables as part of a intr_type.
*/
struct variable_context
{
	typedef std::vector<scope_ref> var_vector_t;

	/**
		Get a list of all the declarations comprising this context.
	*/
	virtual var_vector_t get_local_decls() const = 0;
	
	/**
		Get a list of all the decls with this name. The resulting list could have 0,1,or 2+ entries.
	*/
	virtual var_vector_t get_vars( std::string const & name ) const = 0;
	
	enum get_flags_t : unsigned {
		//only return local items, no parent/extends/continues
		gf_strict_local = 1<<0,
	};
	/**
		Resolves a named type to the actual type. This type must already have been
		added to this scope or a parent scope.
	*/
	virtual shared_ptr<intr_type const> if_get_fun_type( std::string const & name, 
		unsigned get_flags = 0 ) const = 0;
	
};

struct managed_variable_context : public variable_context
{
	virtual var_vector_t get_local_decls() const;
	virtual var_vector_t get_vars( std::string const & name ) const;
	virtual shared_ptr<intr_type const> if_get_fun_type( std::string const & /*name*/, 
		unsigned /*get_flags = 0*/ ) const {
		return shared_ptr<intr_type const>(); 
	}
	
	void add_managed( scope_ref sr ) {
		managed.push_back( sr );
	}
private:
	var_vector_t managed;
};

} //eon leaf

#endif