#ifndef UTIL_VALIDATION_CONTEXT_HPP
#define UTIL_VALIDATION_CONTEXT_HPP

#include <set>

namespace util
{

/**
	The validation_context  allows validation routines to operator on potentially
	recrusive structures.
*/
struct validation_context
{
	std::set<void const*> validating;
	
	bool in( void const * what ) const {
		return validating.find( what ) != validating.end();
	}
};

/**
	A potentially recursive validation should use a scope to mark itself. This ensures
	it is removed once the scope is exited.
*/
struct validation_scope
{
	validation_context & ctx;
	void const * what;
	
	validation_scope( validation_context & ctx, void const * what ) 
		: ctx( ctx )
		, what( what ) {
		ctx.validating.insert( what );
	}
	~validation_scope() {
		ctx.validating.erase( what );
	}
};

} //eon util

#endif