#!/usr/bin/python
"""
	Run a single executable test.
"""
import argparse, os, subprocess, re, sys, codecs, yaml
from shelljob import fs, proc

parser = argparse.ArgumentParser( description = 'Run Leaf executable tests' )
parser.add_argument( 'path', metavar='exe_dir', nargs=1)

args = parser.parse_args()
path = args.path[0].rstrip( '/' )

def test_spec(spec):
	for test in spec['tests']:
		test_spec_item( test )

def test_spec_item(item):
	output, exit_code = proc.call( [ '/tmp/' + exe_name ] + [ arg.encode('utf-8') for arg in item.get( 'args', [] ) ], 
		timeout = 5.0, check_exit_code = False )
	
	if 'output' in item:
		print( "Expect", item['output'] )
		print( "Actual", output )
		if item['output'] != output:
			print( "Failed" )
			sys.exit(1)
	
def test_inline():
	output, exit_code = proc.call( '/tmp/' + exe_name, timeout = 5.0, check_exit_code = False )

	# look for inline test spec
	tested = False
	src = codecs.open( path + "/main.leaf", 'r', 'utf-8' ).read()

	m = re.search( 'OMIT', src )
	if m != None:
		sys.exit(2)
		
	m = re.search( 'EXPECT-RETURN\\(([0-9]+)\\)', src )
	if m != None:
		tested = True
		print( "Check exit code {} == {}".format( exit_code, m.group(1) ) )
		if exit_code != int(m.group(1)):
			print( "Failed" )
			sys.exit(1)
	elif exit_code != 0:
		print( "Invalid exit code: {}".format( exit_code ) )
		sys.exit(1)
			
	m = re.search( 'EXPECT\\(([^\\)]+)\\)', src )
	if m != None:
		tested = True
		print( "Expect: " + m.group(1).encode( 'utf-8') )
		print( "Actual: " + output.encode( 'utf-8' ) )
		if m.group(1) != output:
			print( "Failed" )
			sys.exit(1)
			
	if not tested:
		print( "No tests" )
		sys.exit(1)

def build_exe():
	global exe_name
	exe_name = os.path.basename( path )
	files = fs.find( path, name_regex = '.*leaf' )
	cmd = [ 'build/leaf', '--exe', '/tmp/' + exe_name ] + list(files)
	print(cmd)
	subprocess.check_call(cmd)

build_exe()
tspec = path + '/tests.yaml'
if os.path.exists( tspec ):
	test_spec( yaml.load( codecs.open( tspec, 'r', 'utf-8' ) ) )
else:
	test_inline()
