#include "lang/dump.hpp"
#include "lang/serial.hpp"

#include <boost/program_options.hpp>
namespace po = boost::program_options;
#include <boost/exception/diagnostic_information.hpp>

#include <vector>
#include <fstream>

void load_file( std::string const & name );

int main( int argc, char ** argv ) {

	std::vector<std::string> obj_files;
	
	po::options_description desc( "Options" );
	desc.add_options()
		( "help", "show help" )
		( "obj-file", po::value(&obj_files), "Leaf object files" )
		;
	po::positional_options_description p;
	p.add( "obj-file", -1 );
	
	po::variables_map vm;
	po::store( po::command_line_parser(argc,argv).options(desc).positional(p).run(), vm );
	po::notify( vm );
	
	if( vm.count( "help" ) ) {
		std::cerr << desc;
		return 0;
	}
	
	if( obj_files.empty() ) {
		std::cerr << "requires object file" << std::endl;
		return 1;
	}

	try {
		for( auto & of : obj_files ) {
			load_file( of );
		}
		return 0;
	} catch(...) {
		std::cerr << boost::current_exception_diagnostic_information();
		return 1;
	}

}

void load_file( std::string const & name ) {
	std::ifstream in( name );
	leaf::serial_in sin( in );
	leaf::dump d( std::cout );
	
	for( auto & sitem : sin.get_all_items() ) {
		switch( sitem.type ) {
			case leaf::serial_item::t_invalid:
			case leaf::serial_item::t_end_marker:
				break;
				
			case leaf::serial_item::t_extr_type:
				//these aren't actually of direct interest
				break;
			case leaf::serial_item::t_intr_type:
				if( sitem.v_intr_type->is_custom() 
					|| sitem.v_intr_type->is_distinct() ) {
					d.write_intr_type( sitem.v_intr_type.get(), true );
					d.line_break();
				}
				break;
				
			case leaf::serial_item::t_var_decl:
				d.write_var_decl( *sitem.v_var_decl );
				d.line_break();
				break;
		}
	}
}
