#include "ir/function.hpp"
#include "ir/block.hpp"

#include <set>

namespace ir
{

function::function( compiler_context & cctx, shared_ptr<value> use_value, private_ctor_t )
	: cctx( cctx )
	, anon_count( 0 )
{
	is_export = false;
	
	if( !use_value ) {
		defn_value = make_shared<value>(value::f_global);
	} else {
		STATE_CHECK( !use_value->direct_type );
		defn_value = use_value;
		defn_value->form = value::f_global;
	}
	
	entry_block.reset( new block( *this ) );
}

shared_ptr<value> function::make_temp( shared_ptr<type const> dt )
{
	auto v = make_shared<value>();
	v->form = value::f_temp;
	v->name = create_temp_name();
	v->direct_type = dt;
	return v;
}

std::string function::create_temp_name()
{
	anon_count++;
	char buf[128];
	sprintf( buf, "%%%d", anon_count );
	return buf;
}

namespace
{
	std::set<function const*> validating;
	
	//NOTE: I tried BOOST_SCOPE_EXIT and it fails to compile using just the one header
	struct scope_validating
	{
		function const * what;
		scope_validating( function const * what )
			: what( what )
		{ validating.insert( what ); }
		~scope_validating()
		{ validating.erase( what ); }
	};
}

void function::validate() const
{
	//prevent recursion TODO: this is kind of cheap how this is done (use validation_context.hpp)
	if( validating.find( this ) != validating.end() )
		return;
	scope_validating sv( this );
		
	STATE_CHECK( defn_value );
	defn_value->validate();
	
	for( auto & p : params )
		p.validate();
	for( auto & r : returns )
		r.validate();
	for( auto & c : captures )
		c.validate();
		
	for( auto & l : locals )
		l->validate();
		
	STATE_CHECK( entry_block );
	entry_block->validate();
}

} //eon ir