#include "ir/type_manager.hpp"
#include "ir/type.hpp"
#include "ir/value.hpp"
#include "ir/module.hpp"

#include "platform/platform.hpp"

#define SHOW_DEBUG 0
#include "util/debug.hpp"
#include "lang/dump.hpp"

namespace ir
{

type_manager::type_manager( )
{
	type_boolean = make_shared<type>(type::f_boolean);
	type_empty = make_shared<type>(type::f_tuple);
}

shared_ptr<type const> type_manager::get_integer( int bitsize )
{
	if( !bitsize )
		bitsize = platform::natural_integer_size;
	
	auto it = integer_cache.find( bitsize );
	if( it != integer_cache.end() )
		return it->second;
		
	auto out = make_shared<type>(type::f_integer);
	out->trivial_bitsize = bitsize;
	integer_cache[bitsize] = out;
	return out;
}

shared_ptr<type const> type_manager::get_binary( int bitsize )
{
	if( !bitsize )
		bitsize = platform::natural_integer_size;
		
	auto it = binary_cache.find( bitsize );
	if( it != binary_cache.end() )
		return it->second;
		
	auto out = make_shared<type>(type::f_binary);
	out->trivial_bitsize = bitsize;
	binary_cache[bitsize] = out;
	return out;
}

} //eon ir