#include "parser/source.hpp"

namespace leaf
{

void source::skip_space( unsigned flags )
{
	while( at < end )
	{
		src_char_t next = *at;

		//line comment
		if( next == '/' && (at+1) < end && *(at+1) == '/' ) {
			while( at < end && *at != '\n' && *at != '\r' )
				++at;
			continue;
		}

		//block comment
		if( next == '/' && (at+1) < end && *(at+1) == '*' ) {
			at += 2;
			unsigned depth = 1;
			while( (at+1) < end && depth ) {
				if( *at == '*' && *(at+1) == '/' ) {
					at+=2;
					depth--;
					continue;
				}

				if( *at == '/' && *(at+1) == '*' ) {
					at+=2;
					depth++;
					continue;
				}

				at++;
			}

			if( depth )
				throw_expecting( "end of comment" );
			continue;
		}

		//avoid line-end
		if( next == '\\' ) {
			at++;
			while( at < end && *at != '\n' ) {
				if( !isspace( *at ) )
					throw_unexpected( "data-past-line-continuation" );
				at++;
			}
			if( at == end )
				throw_expecting( "line-contiunation" );
			at++;
			continue;
		}
		
		//line-ends have significance in declaration blocks
		if( next == '\n' && !(flags & skip_line_ends) ) {
			break;
		}
		
		if( isspace( next ) ) {
			at++;
			continue;
		}
		
		break;
	}
}

void source::throw_extend( recoverable_error & ex )
{
	ex << text_at_location( cur_text() );
	if( name.size() )
		ex << except_name( name );
	
	util::source_location loc;
	loc.src = shared_from_this();
	loc.loc = cur_loc();
	ex << util::source_location::error( loc );
	
	throw ex;
}

void source::throw_expecting( std::string const & msg )
{
	recoverable_error re( std::string("Expecting ") + msg );
	throw_extend( re );
}

void source::throw_invalid( std::string const & msg )
{
	recoverable_error re( std::string("Invalid ") + msg );
	throw_extend( re );
}

void source::throw_unexpected( std::string const & msg )
{
	recoverable_error re( std::string("Unexpected ") + msg );
	throw_extend( re );
}

std::string source::cur_text() const
{
	return std::string( at, std::min( end, at + 50 ) );
}

size_t source::line_at( size_t loc ) const
{
	src_char_t const * q = convert_loc( loc );
	size_t line = 1;
	while( q > begin ) {
		q--;
		if( *q == '\n' )
			line++;
	}
	
	return line;
}

size_t source::column_at( size_t loc, size_t tab_size ) const
{
	src_char_t const * q = convert_loc( loc );
	size_t col = 1;
	while( q > begin ) {
		q--;
		if( *q == '\n' )
			break;
		if( *q == '\t' )
			col += tab_size;
		else	
			col++;
	}
	
	return col;
}

src_char_t const * source::convert_loc( size_t loc ) const
{
	PRE_CHECK( loc <= data.size() );
	return begin + loc;
}

} //eon leaf