#define SHOW_DEBUG 0
#include "type_converter_common.hpp"

BOOST_AUTO_TEST_SUITE( type_converter_call )

struct tester_call : public type_converter_tester
{
	void operator()( char const * sto, char const * sfrom, 
		char const * expect )
	{
		expect_test ex;
		ex.add( "To", sto );
		ex.add( "From", sfrom );

		type_converter::convert_t cost;
		try {
			extr_type_ref tog, fromg;
			tog = parse( sto );
			auto & to = tog.fun_cast<intr_type_value_list>();
			fromg = parse( sfrom );
			auto & from = fromg.fun_cast<intr_type_value_list>();

			cost = tcvt.convert_function_call( to, from );
		} catch( ... ) {
			ex.except();
		}
		
		ex.check( cost.okay(), cost.explain_error() );
			
		std::ostringstream out;
		if( cost.conversions )
			dump( out, *cost.conversions );
		ex.check_actual( out.str(), expect );
	}
};

BOOST_AUTO_TEST_CASE(basics)
{
	tester_call t;
	//some basics, not trying to reject all conversion just that they work via function calls
	t( "()","()", "c_pack_value" );
	t( "(:integer)", "(:integer optional)", "c_pack_value {[0]->c_deref_optional}" );
	t( "(:float,:integer)", "(:rational,:(:integer))",
		"c_pack_value {[0]->c_fixate(normal float) [1]->c_collapse}" );
}

BOOST_AUTO_TEST_CASE(optional)
{
	tester_call t;
	t( "( : integer optional )", "()", "c_pack_value {c_unset_optional}" );
	t( "( : float, :float optional )", "(:rational)", "c_pack_value {[0]->c_fixate(normal float) c_unset_optional}" );
}


BOOST_AUTO_TEST_SUITE_END()